#include<stdio.h>
#include<math.h>

/* 二分法でf(x)=0の解を求めるプログラム */

int main(){

  double x_l, x_r, f_l, f_r, x_mean, f_mean, eps = pow(10.0, -14);
  int iter;

  double f(double);

  /* x_lとx_rの初期値 */
  x_l = 0.0;
  x_r = 10.0;

  /* 符号をチェックするためf(x_l)とf(x_r)を計算 */
  f_l = f(x_l);
  f_r = f(x_r);

  /* 二分法ではこれらの符号が違う(積の符号が負) */
  if( f_l * f_r > 0.0 ) {
    fprintf(stderr, "choose other values for x_l and x_r\n");
    return(1);
  }

  // 反復のループ
  for(iter=1;;iter++){

    // 中点の値
    x_mean = (x_l + x_r)/2.0;
    // 中点でのf(x)の値
    f_mean = f(x_mean);

   
    if( f_mean * f_l > 0.0 ) {
      /* 中点でのf(x)の符号がf(x_l)と同じであれば
         x_lを中点で置き換える。f_lも中点での値に置き換える */
      x_l = x_mean;
      f_l = f_mean;
    } else {
      /* そうでない場合、中点でのf(x)の符号はf(x_r)と同じなので
	 x_rを中点で置き換える。f_rも中点での値に置き換える */
      x_r = x_mean;
      f_r = f_mean;
    }

    // 反復の途中経過を出力
    printf(" iter = %d  x_mean = %20.15e  f_mean = %15.10e\n", iter, x_mean, f_mean);

    // x_lとx_rが十分近くなったら、あるいはf(x)が十分に小さくなればループから抜ける
    if( fabs(f_mean) < eps) break;

    // 1000回反復しても終了しない場合はなにかおかしいため強制的に終了する
    if( iter == 1000) {
      fprintf(stderr, "iteration did not converge.\n");
      return(1);
    }
    
  }

  return(0);
  
}

// f(x)の値を返す関数
double f(double x){
  double val;
  val = sin(x/4.0) - cos(x/4.0);
  return(val);

}


