#include<stdio.h>
#include<math.h>

/* Newton法でf(x)=0の解を求めるプログラム */

int main(){

  double x_l, x_r, x_new, f_new, eps = pow(10.0, -14);
  int iter;

  double f(double);
  double dfdx(double);
  
  /* xの境界値 */
  x_l = 0.0;
  x_r = 10.0;

  // xの初期値
  x_new = (x_l + x_r)/2.0;
  
  // 反復のループ
  for(iter=1;;iter++){

    // Newton法で次の点を計算
    x_new = x_new - f(x_new)/dfdx(x_new);
    f_new = f(x_new);

    // 反復の途中経過を出力
    printf(" iter = %d  x_new = %20.15e  f_new = %15.10e\n", iter, x_new, f_new);

    // 境界値より外に出たら計算を終了する
    if(x_new < x_l || x_new > x_r) {
      fprintf(stderr, "x_new out of range\n");
      return(1);
    }
    
    // f_newが十分に小さくなればループから抜ける
    if( fabs(f_new) < eps) break;

    // 1000回反復しても終了しない場合はなにかおかしいため強制的に終了する
    if( iter == 1000) {
      fprintf(stderr, "iteration did not converge.\n");
      return(1);
    }
    
  }

  return(0);
  
}

// f(x)の値を返す関数
double f(double x){
  double val;
  val = sin(x/4.0) - cos(x/4.0);
  return(val);

}

// df/dxの値を返す関数副プログラム
double dfdx(double x){
  double val;
  val = (sin(x/4.0) + cos(x/4.0)) / 4.0;
  return(val);
}
