#include<stdio.h>
#include<math.h>

/* 多次元Newton法でf(x,y)=0, g(x,y)=0の解を求めるプログラム */

int main(){

  double x_new[2], f_new[2], eps = pow(10.0, -14);
  int iter, i, j;

  double jacob[4], det, jinv[2][2];
  
  void f(double[2],double*);
  void dfdx(double[2],double*);
  
  /* xの初期値 */
  x_new[0] = 1.0;
  x_new[1] = 1.0;
  /* f(x)を計算 f_newにはアドレスを渡す*/
  f(x_new, f_new);


  // 反復のループ
  for(iter=1;;iter++){

    // Jacobianを計算。jacobは一次元配列でアドレスを渡す
    dfdx(x_new, jacob);
    
    // Jacobianの逆行列
    det = jacob[0]*jacob[3] - jacob[1]*jacob[2];
    jinv[0][0] = jacob[3]/det;
    jinv[1][1] = jacob[0]/det;
    jinv[0][1] = -jacob[1]/det;
    jinv[1][0] = -jacob[2]/det;

    // Newton法で次の点を計算

    for(i=0;i<2;i++){
      for(j=0;j<2;j++){
	x_new[i] += - jinv[i][j]*f_new[j];
      }
    }
    // 次の点でのf(x)を計算
    f(x_new, f_new);
    
    // 反復の途中経過を出力
    printf(" iter = %d\n", iter);
    printf("x_new = %20.15e %20.15e\n", x_new[0], x_new[1]);
    printf("f_new = %20.15e %20.15e\n", f_new[0], f_new[1]);

    // f_newが十分に小さくなればループから抜ける
    if( sqrt(f_new[0]*f_new[0]+f_new[1]*f_new[1]) < eps) break;

    // 1000回反復しても終了しない場合はなにかおかしいため強制的に終了する
    if( iter == 1000) {
      fprintf(stderr, "iteration did not converge.\n");
      return(1);
    }
    
  }

  return(0);
  
}

// f(x)の値を返す関数 配列の値を返したいのでvalはポインタで宣言しアドレスを受け取る
void f(double x[2], double *val){

  val[0] = x[0]*x[0] + x[1]*x[1] - 10.0;
  val[1] = x[0] - x[1]*x[1]*x[1];
  
}

// df/dxの値を返す関数
// df/dxは一次元配列で定義
void dfdx(double x[2], double *val){

  val[0] = 2.0*x[0];
  val[1] = 2.0*x[1];
  val[2] = 1.0;
  val[3] = -3.0*x[1]*x[1];

}
