#include<stdio.h>
#include<math.h>

/* Broyden法でf(x,y)=0, g(x,y)=0の解を求めるプログラム */

int main(){

  double x_bro[2], f_bro[2], f_bro2[2], eps = pow(10.0, -14);
  int iter, i, j;

  double binv[2][2], denom, nom1[2], nom2[2];

  double deltaf_bro[2], deltax_bro[2];
  
  void f(double[2],double*);
  
  /* xの初期値 */
  x_bro[0] = 0.0;
  x_bro[1] = 0.0;
  /* 初期値でのf(x)を計算 */
  f(x_bro, f_bro);

  printf("initial x   = %15.5f %15.5f\n", x_bro[0], x_bro[1]);
  printf("initial f(x)= %15.5f %15.5f\n", f_bro[0], f_bro[1]);

  // Jacobianの逆行列の初期値(単位行列に設定)
  binv[0][0] = 1.0;
  binv[1][1] = 1.0;
  binv[0][1] = 0.0;
  binv[1][0] = 0.0;
  
  // 反復のループ
  for(iter=1;;iter++){

    // Newton法で次の点を計算
    // \Delta x^{(k)}
    for(i=0;i<2;i++){
      deltax_bro[i] = 0.0;
    }    
    for(i=0;i<2;i++){
      for(j=0;j<2;j++){
	deltax_bro[i] -= binv[i][j]*f_bro[j];
      }
    }

    // x^{(k+1)}
    for(i=0;i<2;i++){
      x_bro[i] += deltax_bro[i];
    }

    // f(x^{(k+1)})
    f(x_bro, f_bro2);

    // \Delta f
    for(i=0;i<2;i++){
      deltaf_bro[i] = f_bro2[i] - f_bro[i];     
    }

    // f_broにf(x^{(k+1)})を代入して次の反復計算で用いる
    for(i=0;i<2;i++){
      f_bro[i] =f_bro2[i];
    }

    // B^{-1}行列の分母
    denom = 0.0;
    for(i=0;i<2;i++){
      for(j=0;j<2;j++){
	denom += deltax_bro[i] * binv[i][j] * deltaf_bro[j];
      }
    }

    // B^{-1}行列の分子左側
    for(i=0;i<2;i++){
      nom1[i] = deltax_bro[i];
    }
    for(i=0;i<2;i++){
      for(j=0;j<2;j++){
	nom1[i] -= binv[i][j] * deltaf_bro[j];
      }
    }

    // B^{-1}行列の分子右側
    for(i=0;i<2;i++){
      nom2[i] = 0.0;
    }
    for(i=0;i<2;i++){
      for(j=0;j<2;j++){
	nom2[i] += deltax_bro[j] * binv[j][i];
      }
    }

    // B^{(k+1)}^{-1}の計算
    for(i=0;i<2;i++){
      for(j=0;j<2;j++){
	binv[i][j] += nom1[i]*nom2[j]/denom;
      }
    }
    
	
    // 反復の途中経過を出力
    printf(" iter = %d\n", iter);
    printf("x_bro = %20.15e %20.15e\n", x_bro[0], x_bro[1]);
    printf("f_bro = %20.15e %20.15e\n", f_bro[0], f_bro[1]);

    // f_broが十分に小さくなればループから抜ける
    if( sqrt(f_bro[0]*f_bro[0]+f_bro[1]*f_bro[1]) < eps) break;

    // 1000回反復しても終了しない場合はなにかおかしいため強制的に終了する
    if( iter == 1000) {
      fprintf(stderr, "iteration did not converge.\n");
      return(1);
    }
    
  }

  return(0);
  
}

// f(x)の値を返す関数
void f(double x[2], double *val){

  val[0] = x[0]*x[0] + x[1]*x[1] - 10.0;
  val[1] = x[0] - x[1]*x[1]*x[1];
  
}

